==> tuain-ecosystem-lib-0.0.6.tgz: package/lib/ecosystem/prices.js <==
const PRODPRICE_COLLECTION = 'ProductPriceTypes';
const GROUPPRICE_COLLECTION = 'GroupProductPrices';
const PRICEHIST_COLLECTION = 'productPricesHistory';

const moduleErrors = {
  createProductPrice: {
    alreadyExist: ['01', 'La definición de precio del producto ya existe'],
  },
  findProductPrice: {
    notFound: ['01', 'Definición de precio del producto no encontrada'],
  },
  modifyProductPrice: {
    alreadyActive: ['01', 'Definición de precio del producto activa'],
    alreadyInactive: ['02', 'Definición de precio del producto no activa'],
  },
  groupProductPrice: {
    alreadyExist: ['01', 'Tarifa de producto a grupo ya existe'],
    notFound: ['01', 'Tarifa de producto a grupo no encontrada'],
  }
};

class PriceManager {

  constructor(getDb, logger, errMgr, priceQueries) {
    this.getDb = getDb;
    this.logger = logger;
    this.errMgr = errMgr;
    this.priceQueries = priceQueries;
    this.errMgr.addModuleSet('prices', moduleErrors);

  }

  async createProductPriceType(prodPriceData) {
    const { name } = prodPriceData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findByName(name));
    if (prodPriceDetail) {
      const errorDetail = `Precio de producto ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newProdPriceData = { ...prodPriceData, enabled: false, creation: new Date() };
    const actionResult = await prodPriceCol.insertOne(newProdPriceData);
    this.logger.debug(`Creación de una nueva tarifa de producto ${newProdPriceData} con resultado ${actionResult?.insertedId}`);
    return [null, { envId: actionResult?.insertedId }];
  }

  async updateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    const updateData = { ...reqData };
    delete updateData.prodPriceId;
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación de una tarifa de producto ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    if (prodPriceDetail.enabled) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyRelation.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación de la relación ${prodPriceId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!prodPriceDetail.enabled) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyRelation.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del precio del producto ${prodPriceId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const actionResult = await prodPriceCol.deleteOne(this.priceQueries.findById(prodPriceId));
    this.logger.debug(`Eliminación del precio del producto ${prodPriceId} con resultado ${actionResult?.result?.n}`);
    const result = actionResult?.result?.n > 0;
    return [null, { result }];
  }

  async defineGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId, priceValue, author } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol.findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (groupProdPriceDetail) {
      const errorDetail = `Ya existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newProdPriceData = { ...reqData, enabled: false, creation: new Date() };
    const auditGroupPriceChange = {
      productPriceTypeId, productId, subproductId, groupId, author, priceValue, priceStartDate: new Date(), priceEndDate: null
    };
    auditGroupProdPriceCol.insertOne(auditGroupPriceChange);
    const actionResult = await groupProdPriceCol.insertOne(newProdPriceData);
    this.logger.debug(`Creación de una nueva tarifa de producto ${newProdPriceData} con resultado ${actionResult?.insertedId}`);
    return [null, { groupProductPriceId: actionResult?.insertedId }];
  }

  async updateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId, priceValue, author } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol.findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const updateData = { productId, subproductId, groupId, priceValue, author, update: new Date() };
    groupProdPriceCol.updateOne(this.priceQueries.findById(groupProdPriceDetail._id), this.priceQueries.updateObj(updateData));
    const auditGroupPriceChange = {
      productPriceTypeId, productId, subproductId, groupId, author, priceValue, priceStartDate: new Date(), priceEndDate: null
    };
    auditGroupProdPriceCol.insertOne(auditGroupPriceChange);
    auditGroupProdPriceCol.updateOne(priceQueries
      .findAuditGroupProdPrice(productPriceTypeId, productId, subproductId, groupId),
      this.priceQueries.updateObj({ priceEndDate: new Date() }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];
  }

  async activateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol.findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    groupProdPriceCol.updateOne(this.priceQueries.findById(groupProdPriceDetail._id), this.priceQueries.updateObj({ enabled: true }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];
  }

  async inctivateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol.findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    groupProdPriceCol.updateOne(this.priceQueries.findById(groupProdPriceDetail._id), this.priceQueries.updateObj({ enabled: false }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];

  }

  async deleteGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol.findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    if (groupProdPriceDetail.enabled) {
      const errorDetail = `La tarfa del producto para el grupo ${{ productId, subproductId, groupId }} está activa`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await groupProdPriceCol.deleteOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    this.logger.debug(`Eliminación de la tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`);
    const result = actionResult?.result?.n > 0;
    auditGroupProdPriceCol.updateOne(priceQueries
      .findAuditGroupProdPrice(productPriceTypeId, productId, subproductId, groupId),
      this.priceQueries.updateObj({ priceEndDate: new Date() }));
    return [null, { result }];
  }
}

module.exports = PriceManager;

==> tuain-ecosystem-lib-0.0.9.tgz: package/lib/ecosystem/prices.js <==
const PRODPRICE_COLLECTION = 'ProductPriceTypes';
const GROUPPRICE_COLLECTION = 'GroupProductPrices';
const PRICEHIST_COLLECTION = 'productPricesHistory';

const moduleErrors = {
  createProductPrice: {
    alreadyExist: ['01', 'La definición de precio del producto ya existe'],
  },
  findProductPrice: {
    notFound: ['01', 'Definición de precio del producto no encontrada'],
  },
  modifyProductPrice: {
    alreadyActive: ['01', 'Definición de precio del producto activa'],
    alreadyInactive: ['02', 'Definición de precio del producto no activa'],
  },
  groupProductPrice: {
    alreadyExist: ['01', 'Tarifa de producto a grupo ya existe'],
    notFound: ['01', 'Tarifa de producto a grupo no encontrada'],
  },
};

class PriceManager {
  constructor(getDb, logger, errMgr, priceQueries) {
    this.getDb = getDb;
    this.logger = logger;
    this.errMgr = errMgr;
    this.priceQueries = priceQueries;
    this.errMgr.addModuleSet('prices', moduleErrors);
  }

  async createProductPriceType(prodPriceData) {
    const { name } = prodPriceData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findByName(name));
    if (prodPriceDetail) {
      const errorDetail = `Precio de producto ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newProdPriceData = { ...prodPriceData, enabled: false, creation: new Date() };
    const actionResult = await prodPriceCol.insertOne(newProdPriceData);
    this.logger.debug(`Creación de una nueva tarifa de producto ${newProdPriceData} con resultado ${actionResult?.insertedId}`);
    return [null, { envId: actionResult?.insertedId }];
  }

  async updateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    const updateData = { ...reqData };
    delete updateData.prodPriceId;
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación de una tarifa de producto ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    if (prodPriceDetail.enabled) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyRelation.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación de la relación ${prodPriceId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const prodPriceDetail = await prodPriceCol.findOne(this.priceQueries.findById(prodPriceId));
    if (!prodPriceDetail) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProductPrice.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!prodPriceDetail.enabled) {
      const errorDetail = `Precio de producto con Id ${prodPriceId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyRelation.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await prodPriceCol.updateOne(this.priceQueries.findById(prodPriceId),
      this.priceQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del precio del producto ${prodPriceId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteProductPriceType(reqData) {
    const { prodPriceId } = reqData;
    const prodPriceCol = this.getDb().collection(PRODPRICE_COLLECTION);
    const actionResult = await prodPriceCol.deleteOne(this.priceQueries.findById(prodPriceId));
    this.logger.debug(`Eliminación del precio del producto ${prodPriceId} con resultado ${actionResult?.result?.n}`);
    const result = actionResult?.result?.n > 0;
    return [null, { result }];
  }

  async defineGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId, priceValue, author } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol
      .findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (groupProdPriceDetail) {
      const errorDetail = `Ya existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newProdPriceData = { ...reqData, enabled: false, creation: new Date() };
    const auditGroupPriceChange = {
      productPriceTypeId,
      productId,
      subproductId,
      groupId,
      author,
      priceValue,
      priceStartDate: new Date(),
      priceEndDate: null,
    };
    auditGroupProdPriceCol.insertOne(auditGroupPriceChange);
    const actionResult = await groupProdPriceCol.insertOne(newProdPriceData);
    this.logger.debug(`Creación de una nueva tarifa de producto ${newProdPriceData} con resultado ${actionResult?.insertedId}`);
    return [null, { groupProductPriceId: actionResult?.insertedId }];
  }

  async updateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId, priceValue, author } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol
      .findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const updateData = { productId, subproductId, groupId, priceValue, author, update: new Date() };
    groupProdPriceCol.updateOne(this.priceQueries
      .findById(groupProdPriceDetail._id), this.priceQueries.updateObj(updateData));
    const auditGroupPriceChange = {
      productPriceTypeId,
      productId,
      subproductId,
      groupId,
      author,
      priceValue,
      priceStartDate: new Date(),
      priceEndDate: null,
    };
    auditGroupProdPriceCol.insertOne(auditGroupPriceChange);
    auditGroupProdPriceCol.updateOne(this.priceQueries
      .findAuditGroupProdPrice(productPriceTypeId, productId, subproductId, groupId),
    this.priceQueries.updateObj({ priceEndDate: new Date() }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];
  }

  async activateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol
      .findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    groupProdPriceCol.updateOne(this.priceQueries
      .findById(groupProdPriceDetail._id), this.priceQueries.updateObj({ enabled: true }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];
  }

  async inctivateGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol
      .findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    groupProdPriceCol.updateOne(this.priceQueries
      .findById(groupProdPriceDetail._id), this.priceQueries.updateObj({ enabled: false }));
    return [null, { groupProductPriceId: groupProdPriceDetail.id }];
  }

  async deleteGroupProductPrice(reqData) {
    const { productPriceTypeId, productId, subproductId, groupId } = reqData;
    const groupProdPriceCol = this.getDb().collection(GROUPPRICE_COLLECTION);
    const auditGroupProdPriceCol = this.getDb().collection(PRICEHIST_COLLECTION);
    const groupProdPriceDetail = await groupProdPriceCol
      .findOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    if (!groupProdPriceDetail) {
      const errorDetail = `No existe una definición de tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    if (groupProdPriceDetail.enabled) {
      const errorDetail = `La tarfa del producto para el grupo ${{ productId, subproductId, groupId }} está activa`;
      const errorObj = this.errMgr.get(moduleErrors.groupProductPrice.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await groupProdPriceCol
      .deleteOne(this.priceQueries.findGroupProdPrice(productPriceTypeId, productId, subproductId, groupId));
    this.logger.debug(`Eliminación de la tarfa del producto para el grupo ${{ productId, subproductId, groupId }}`);
    const result = actionResult?.result?.n > 0;
    auditGroupProdPriceCol.updateOne(this.priceQueries
      .findAuditGroupProdPrice(productPriceTypeId, productId, subproductId, groupId),
    this.priceQueries.updateObj({ priceEndDate: new Date() }));
    return [null, { result }];
  }
}

module.exports = PriceManager;
